@@ -18,11 +18,22 @@
 namespace mtx::iso639 {
 
 struct language_t {
-  std::string const english_name, iso639_2_code, iso639_1_code, terminology_abbrev;
+  std::string const english_name, alpha_3_code, alpha_2_code, terminology_abbrev;
+  bool is_part_of_iso639_2{};
+
+  language_t(std::string &&p_english_name, std::string &&p_alpha_3_code, std::string &&p_alpha_2_code, std::string &&p_terminology_abbrev, bool p_is_part_of_iso639_2)
+    : english_name{std::move(p_english_name)}
+    , alpha_3_code{std::move(p_alpha_3_code)}
+    , alpha_2_code{std::move(p_alpha_2_code)}
+    , terminology_abbrev{std::move(p_terminology_abbrev)}
+    , is_part_of_iso639_2{p_is_part_of_iso639_2}
+  {
+  }
 };
 
-extern std::vector<language_t> const g_languages;
+extern std::vector<language_t> g_languages;
 
+void init();
 std::optional<language_t> look_up(std::string const &s, bool allow_short_english_names = false);
 void list_languages();
 
