@@ -16,6 +16,7 @@
 #include "common/ebml.h"
 #include "common/iso639.h"
 #include "common/list_utils.h"
+#include "common/regex.h"
 #include "common/strings/formatting.h"
 #include "common/strings/parsing.h"
 #include "common/translation.h"
@@ -202,11 +203,9 @@ void
 extract_cli_parser_c::set_simple_language() {
   assert_mode(options_c::em_chapters);
 
-  auto language_idx = mtx::iso639::look_up(m_next_arg);
-  if (!language_idx)
+  m_current_mode->m_simple_chapter_language = mtx::bcp47::language_c::parse(m_next_arg);
+  if (!m_current_mode->m_simple_chapter_language.has_valid_iso639_code())
     mxerror(fmt::format(Y("'{0}' is neither a valid ISO 639-2 nor a valid ISO 639-1 code. See 'mkvmerge --list-languages' for a list of all languages and their respective ISO 639-2 codes.\n"), m_next_arg));
-
-  m_current_mode->m_simple_chapter_language = mtx::iso639::g_languages[*language_idx].iso639_2_code;
 }
 
 std::optional<options_c::extraction_mode_e>
@@ -339,10 +338,10 @@ extract_cli_parser_c::add_extraction_spec() {
       && (options_c::em_attachments   != m_current_mode->m_extraction_mode))
     mxerror(fmt::format(Y("Unrecognized command line option '{0}'.\n"), m_current_arg));
 
-  std::regex s_track_id_re("^(\\d+)(:(.+))?$");
+  mtx::regex::jp::Regex s_track_id_re{"^(\\d+)(?::(.+))?$"};
 
-  std::smatch matches;
-  if (!std::regex_search(m_current_arg, matches, s_track_id_re)) {
+  mtx::regex::jp::VecNum matches;
+  if (!mtx::regex::match(m_current_arg, matches, s_track_id_re)) {
     if (options_c::em_attachments == m_current_mode->m_extraction_mode)
       mxerror(fmt::format(Y("Invalid attachment ID/file name specification in argument '{0}'.\n"), m_current_arg));
     else
@@ -351,15 +350,15 @@ extract_cli_parser_c::add_extraction_spec() {
 
   track_spec_t track;
 
-  mtx::string::parse_number(matches[1].str(), track.tid);
+  mtx::string::parse_number(matches[0][1], track.tid);
 
   if (m_used_tids[m_current_mode->m_extraction_mode][track.tid])
     mxerror(fmt::format(Y("The ID '{0}' has already been used for another destination file.\n"), track.tid));
   m_used_tids[m_current_mode->m_extraction_mode][track.tid] = true;
 
   std::string output_file_name;
-  if (matches[3].matched)
-    output_file_name = matches[3].str();
+  if (!matches[0][2].empty())
+    output_file_name = matches[0][2];
 
   if (output_file_name.empty()) {
     if (options_c::em_attachments == m_current_mode->m_extraction_mode)
@@ -378,6 +377,23 @@ extract_cli_parser_c::add_extraction_spec() {
   set_default_values();
 }
 
+void
+extract_cli_parser_c::check_for_identical_source_and_destination_file_names() {
+  if (m_options.m_file_name.empty())
+    return;
+
+  auto source_file_name = bfs::absolute(bfs::path{m_options.m_file_name});
+
+  for (auto const &mode_option : m_options.m_modes) {
+    if (!mode_option.m_output_file_name.empty() && (bfs::absolute(bfs::path{mode_option.m_output_file_name}) == source_file_name))
+      mxerror(fmt::format(Y("The name of one of the destination files is the same as the name of the source file ({0}).\n"), source_file_name.string()));
+
+    for (auto const &track_spec : mode_option.m_tracks)
+      if (!track_spec.out_name.empty() && (bfs::absolute(bfs::path{track_spec.out_name}) == source_file_name))
+        mxerror(fmt::format(Y("The name of one of the destination files is the same as the name of the source file ({0}).\n"), source_file_name.string()));
+  }
+}
+
 options_c
 extract_cli_parser_c::run() {
   init_parser();
@@ -386,6 +402,8 @@ extract_cli_parser_c::run() {
 
   m_options.merge_tracks_and_timestamps_targets();
 
+  check_for_identical_source_and_destination_file_names();
+
   if (m_debug)
     m_options.dump();
 
