@@ -4,7 +4,7 @@
 
    Distributed under the GPL v2
    see the file COPYING for details
-   or visit http://www.gnu.org/copyleft/gpl.html
+   or visit https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 
    MPEG ES (elementary stream) demultiplexer module
 
@@ -76,7 +76,7 @@ mpeg_es_reader_c::probe_file() {
         gop_start_code_found = true;
 
       else if (MPEGVIDEO_EXT_START_CODE == value)
-        gop_start_code_found = true;
+        ext_start_code_found = true;
 
       else if ((MPEGVIDEO_FIRST_SLICE_START_CODE <= value) && (MPEGVIDEO_LAST_SLICE_START_CODE >= value))
         ++num_slice_start_codes_found;
@@ -127,7 +127,7 @@ mpeg_es_reader_c::read_headers() {
     interlaced                  = !seq_hdr.progressiveSequence;
     width                       = seq_hdr.width;
     height                      = seq_hdr.height;
-    frame_rate                  = seq_hdr.progressiveSequence ? seq_hdr.frameOrFieldRate : seq_hdr.frameOrFieldRate * 2.0;
+    frame_rate                  = seq_hdr.frameRate;
     aspect_ratio                = seq_hdr.aspectRatio;
 
     if ((0 >= aspect_ratio) || (1 == aspect_ratio))
@@ -136,12 +136,6 @@ mpeg_es_reader_c::read_headers() {
       dwidth = (int)(height * aspect_ratio);
     dheight = height;
 
-    MPEGChunk *raw_seq_hdr = parser.GetRealSequenceHeader();
-    if (raw_seq_hdr)
-      m_ti.m_private_data = memory_c::clone(raw_seq_hdr->GetPointer(), raw_seq_hdr->GetSize());
-    else
-      m_ti.m_private_data.reset();
-
     mxverb(2, fmt::format("mpeg_es_reader: version {0} width {1} height {2} FPS {3} AR {4}\n", version, width, height, frame_rate, aspect_ratio));
 
   } catch (mtx::mm_io::exception &) {
