@@ -22,6 +22,7 @@
 #include "common/mp3.h"
 #include "common/mpeg1_2.h"
 #include "common/mpeg4_p2.h"
+#include "common/regex.h"
 #include "common/strings/formatting.h"
 #include "common/truehd.h"
 #include "input/r_mpeg_ps.h"
@@ -80,7 +81,7 @@ mpeg_ps_reader_c::read_headers() {
   try {
     uint8_t byte;
 
-    if (!m_ti.m_disable_multi_file && std::regex_search(bfs::path{m_ti.m_fname}.filename().string(), std::regex{"^vts_\\d+_\\d+", std::regex_constants::icase})) {
+    if (!m_ti.m_disable_multi_file && mtx::regex::match(bfs::path{m_ti.m_fname}.filename().string(), mtx::regex::jp::Regex{"^vts_\\d+_\\d+", "i"})) {
       m_in.reset();               // Close the source file first before opening it a second time.
       m_in = mm_multi_file_io_c::open_multi(m_ti.m_fname, false);
     }
@@ -670,13 +671,13 @@ mpeg_ps_reader_c::new_stream_v_mpeg_1_2(mpeg_ps_id_t id,
   track->v_version                = m2v_parser->GetMPEGVersion();
   track->v_width                  = seq_hdr.width;
   track->v_height                 = seq_hdr.height;
-  track->v_frame_rate             = seq_hdr.progressiveSequence ? seq_hdr.frameOrFieldRate : seq_hdr.frameOrFieldRate * 2.0;
+  track->v_frame_rate             = seq_hdr.frameRate;
   track->v_aspect_ratio           = seq_hdr.aspectRatio;
-  track->timestamp_b_frame_offset = 1000000000ll * num_leading_b_fields / seq_hdr.frameOrFieldRate / 2;
+  track->timestamp_b_frame_offset = 1000000000ll * num_leading_b_fields / (seq_hdr.frameRate * 2);
 
   mxdebug_if(m_debug_timestamps,
              fmt::format("Leading B fields {0} rate {1} progressive? {2} calculated_offset {3} found_i? {4} found_non_b? {5}\n",
-                         num_leading_b_fields, seq_hdr.frameOrFieldRate, !!seq_hdr.progressiveSequence, track->timestamp_b_frame_offset, found_i_frame, found_non_b_frame));
+                         num_leading_b_fields, seq_hdr.frameRate, !!seq_hdr.progressiveSequence, track->timestamp_b_frame_offset, found_i_frame, found_non_b_frame));
 
   if ((0 >= track->v_aspect_ratio) || (1 == track->v_aspect_ratio))
     track->v_dwidth = track->v_width;
@@ -684,12 +685,6 @@ mpeg_ps_reader_c::new_stream_v_mpeg_1_2(mpeg_ps_id_t id,
     track->v_dwidth = (int)(track->v_height * track->v_aspect_ratio);
   track->v_dheight  = track->v_height;
 
-  MPEGChunk *raw_seq_hdr = m2v_parser->GetRealSequenceHeader();
-  if (raw_seq_hdr) {
-    track->raw_seq_hdr      = (unsigned char *)safememdup(raw_seq_hdr->GetPointer(), raw_seq_hdr->GetSize());
-    track->raw_seq_hdr_size = raw_seq_hdr->GetSize();
-  }
-
   track->use_buffer(128000);
 }
 
@@ -1241,7 +1236,6 @@ mpeg_ps_reader_c::create_packetizer(int64_t id) {
     if (track->codec.is(codec_c::type_e::V_MPEG12)) {
       generic_packetizer_c *m2vpacketizer;
 
-      m_ti.m_private_data = memory_c::clone(track->raw_seq_hdr, track->raw_seq_hdr_size);
       m2vpacketizer       = new mpeg1_2_video_packetizer_c(this, m_ti, track->v_version, track->v_frame_rate, track->v_width, track->v_height,
                                                            track->v_dwidth, track->v_dheight, false);
       track->ptzr         = add_packetizer(m2vpacketizer);
