@@ -683,7 +683,7 @@ ogm_reader_c::identify() {
     info = mtx::id::info_c{};
 
     info.set(mtx::id::number,   dmx->serialno);
-    info.add(mtx::id::language, dmx->language.get_iso639_2_code());
+    info.add(mtx::id::language, dmx->language.get_iso639_alpha_3_code());
 
     if (!dmx->title.empty() && !dmx->ms_compat)
       info.add(mtx::id::track_name, dmx->title);
@@ -778,7 +778,7 @@ ogm_reader_c::handle_chapters(mtx::tags::vorbis_comments_t const &comments) {
     auto const &sync = mtx::includes(m_ti.m_timestamp_syncs, track_info_c::chapter_track_id) ? m_ti.m_timestamp_syncs[track_info_c::chapter_track_id]
                      : mtx::includes(m_ti.m_timestamp_syncs, track_info_c::all_tracks_id)    ? m_ti.m_timestamp_syncs[track_info_c::all_tracks_id]
                      :                                                                         timestamp_sync_t{};
-    mtx::chapters::adjust_timestamps(*m_chapters, sync.displacement, sync.numerator, sync.denominator);
+    mtx::chapters::adjust_timestamps(*m_chapters, sync.displacement, sync.factor);
 
   } catch (...) {
     m_exception_parsing_chapters = true;
